package um.di.lmf.demo;

import um.di.lmf.security.*;
import um.di.lmf.enote.*;

import iaik.security.rsa.*;
import iaik.asn1.*;
import iaik.x509.*;
import iaik.utils.*;
import java.io.*;
import java.math.*;

/**
 * This class demonstrates the use of the enote package classes. Look at the code.
 */
public class ENoteDemo {
    static public void main(String args[])  {

	try {
	    // Build the demo cert
	    ByteArrayInputStream bais = new ByteArrayInputStream(demo_cert.getBytes());
	    Base64InputStream b64is = new Base64InputStream(bais);
	    X509Certificate bank_cert=new X509Certificate(b64is);

	    // Build the demo private key
	    bais = new ByteArrayInputStream(demo_key.getBytes());
	    b64is = new Base64InputStream(bais);
	    byte[]key_bytes=new byte[bais.available()];
	    bais.read(key_bytes);
	    RSAPrivateKey bank_key = RSAPrivateKey.parse(key_bytes);

	    // Create a new order for a note. The OpenOrder class encloses all the information.
	    OpenOrder oorder=new OpenOrder(new String(id),5,10,bank_cert);

	    // Test the ASN1 encoding/decoding methods.
	    oorder.decode(oorder.toASN1Object());

	    // Create an object that includes a blinded hash and the open order.
	    // The blinded hash applies only to the public part of the OpenOrder (the enclosed Order).
	    // This is what is sent to the bank for those orders which are discarded.
	    OpenBlindedOrder obo=new OpenBlindedOrder(oorder);

	    // Test the ASN1 encoding/decoding methods
	    obo.decode(obo.toASN1Object());

	    // Retrieve an object that includes only the blinded hash.
	    // This is what is sent to the bank initially.
	    BlindedOrder bo=new BlindedOrder();
	    bo.decode(obo.toBlindedOrderASN1Object());

	    // Sign the blinded hash. This is done by the bank.
	    byte[] blinded_signature=RSABlindSignature.sign(bo.getBlindedHash(), bank_key);

	    // Unblind the signature. This is done by the note owner.
	    byte[] signature = RSABlindSignature.unblind(blinded_signature, obo.getBlindingFactor(), new RSAPublicKey(bank_cert.getPublicKey().getEncoded()));

	    // Obtain only the public part of the order, to build the ENote.
	    Order order = new Order();
	    order.decode(oorder.toOrderASN1Object());

	    // Build the ENote.
	    ENote enote = new ENote(order,signature);

	    // Test the ASN1 encoding/decoding methods.
	    enote.decode(enote.toASN1Object());

	    // Build an open ENote to prove to the payee that we own the ENote.
	    boolean[]bits=new boolean[oorder.getNPairs()];
	    OpenENote oenote = new OpenENote(oorder,enote,bits);

	    // Test the ASN1 encoding/decoding methods
	    oenote.decode(oenote.toASN1Object());

	    //Verify the digital signature on the ENote.
	    boolean res=RSABlindSignature.verify(new ASN1(order.toASN1Object()).toByteArray(),enote.getSignature(),new RSAPublicKey(bank_cert.getPublicKey().getEncoded()));
	    if (res) {
		System.out.println("Signature OK");
	    }
	    else {
		System.out.println("Signature Not OK");
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	
    }

    static private String id = "Identidade do titular da nota: secreto";

    static private String demo_cert ="MIICzzCCAjigAwIBAgIBHzANBgkqhkiG9w0BAQQFADBEMQswCQYDVQQGEwJQVDES\nMBAGA1UEChMJREktVU1JTkhPMQ8wDQYDVQQLEwZDQTAzMDQxEDAOBgNVBAMTB0NB\nLVJPT1QwHhcNMDMxMjAzMTIwNzA0WhcNMDQxMjAyMTIwNzA0WjBLMQswCQYDVQQG\nEwJQVDESMBAGA1UEChMJREktVU1JTkhPMQ8wDQYDVQQLEwZDQTAzMDQxFzAVBgNV\nBAMTDjE5Mi4xNjguMTAuMjM0MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCk\nKg6sRxOgKGXG5H6vzlxGAs0PFuPhNFWhz61A702Ntw8Qhq9nv512a7Im6OZS/5BF\n2qNjVvliM5Nr05wwdcEa+CRg/x3YieyKhIISYMfHyS/QhxtPGO0cYVyFsNgknafW\n8wNfoo8HJ4Vnr5t+eLubeHkg7A/166D4tJ4Aza7GXwIDAQABo4HJMIHGMAkGA1Ud\nEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5TU0wgR2VuZXJhdGVkIENlcnRpZmlj\nYXRlMB0GA1UdDgQWBBQAz3mXj8oIUxOBLs4LTkdnmSAsUTBsBgNVHSMEZTBjgBQr\n2lsBetN1DZ75kRV+vuey7J5GMaFIpEYwRDELMAkGA1UEBhMCUFQxEjAQBgNVBAoT\nCURJLVVNSU5ITzEPMA0GA1UECxMGQ0EwMzA0MRAwDgYDVQQDEwdDQS1ST09UggEA\nMA0GCSqGSIb3DQEBBAUAA4GBADzwudBv7bApnlS2Jzs/gXaZd01ExK85vOyANN9j\nxddWICHQIpb/UfLEEnjF6XmNLp91tKoGYH0jqmiS2/MfNwQ+fN4AlSxanhxF0qI8\nD2apTdiXSN/x16A1QBjJ8S4k0HUJJiw/B0q2zPwlJC3v5Nk8s3J1tiyoijNwhoDF\nkMVC";

    static private String demo_key = "MIICXgIBAAKBgQCkKg6sRxOgKGXG5H6vzlxGAs0PFuPhNFWhz61A702Ntw8Qhq9nv512a7Im6OZS/5BF2qNjVvliM5Nr05wwdcEa+CRg/x3YieyKhIISYMfHyS/QhxtP\nGO0cYVyFsNgknafW8wNfoo8HJ4Vnr5t+eLubeHkg7A/166D4tJ4Aza7GXwIDAQAB\nAoGBAI57VkreUcFS7qsGjJIoCFxXeATduCExXBVZ+6R9TpFuTavpSfpBC/jJFlHB\nGI728xA96EuDJGXAH/aZLlm9CkDq++AVCq49occh7EGvn/IwV05yTLiu8PoPDiEd\nP1hW5vqNt/BEy5946oPWI25gVyLDrloBe7eaScHIuQISAUdJAkEA1N8M3uB52LgL\nhZh9Q13Lz4Z2JNELWcmHeY4lVo7fIj+vZOoHDoW72/vm/f2R3YvPsStYL2nwxcB3\n3S46mdliUwJBAMVsuzVnGTExmB/D5oge9ta3afnTSmwuNk6F5u27zPe9WpEG1cKD\nxdzV1+m4cInciwGhpCN5u8v4i5iLu6hO4kUCQQCrfl7iF7pSLEOsB2OzKBeQQuVf\nV8JrPX6VIT1XdJazAteM1zWEL51e5pv7t5MSBxb1xtOurwy4ZAys9hdj5svdAkEA\nh/l1jzhtH3a7AdXZZ5N8CzLUaN7f+VZyzO5ouQCKEW93ez+r5ob/tjQKu8zupeUi\nUINWG6DyeW+8JdrH6wo5yQJAOXKYJnvqN8UKIC9cKJJJ4ALmBi5fsDHcF38JM0Wd\nCGrVUzQIQHwLYvqbMFiIllyyCBEWDUls8zfZXRnxOJvFZg==";

}