package um.di.lmf.enote;

import iaik.asn1.*;
import java.math.*;
import java.util.*;
import java.security.*;
import java.security.interfaces.*;

import um.di.lmf.enote.*;
import um.di.lmf.security.*;

/**
 * This class represents a digital cash note blinded order, used in the withdrawal protocol
 */

public class OpenBlindedOrder extends BlindedOrder implements ASN1Type {
    
    /**
     * This is the constructor that should be used before building an object with decode.
     */
    public OpenBlindedOrder () {
    }

    /**
     * This constructor builds an open blinded order from an open order.
     * The super class reveals only the blinded hash.
     */
    public OpenBlindedOrder(OpenOrder oorder) throws NoSuchAlgorithmException, RSABlindSignatureException, CodingException {
	SecureRandom sr=SecureRandom.getInstance("SHA1PRNG");
	sr.setSeed(Calendar.getInstance().get(Calendar.MILLISECOND ));
	
	this.oorder=oorder;
	
	do{
	    blinding_factor=new BigInteger(160,sr);
	} while (! blinding_factor.gcd(((RSAPublicKey) oorder.getBankCertificate().getPublicKey()).getModulus()).equals(BigInteger.ONE));

	blinded_hash=RSABlindSignature.blind(new ASN1(oorder.toOrderASN1Object()).toByteArray(),blinding_factor,(RSAPublicKey) oorder.getBankCertificate().getPublicKey());
    }
    
    /**
     * Returns the blinding_factor
     */
    public BigInteger getBlindingFactor() {
	return blinding_factor;
    }
    
    /**
     * Returns the enclosed open order
     */
    public OpenOrder getOpenOrder() {
	return oorder;
    }

   /**
     * Creates an ASN1Object of the enclosed BlindedOrder object
     */
    public ASN1Object toBlindedOrderASN1Object() throws CodingException {
	return super.toASN1Object();
    }

    /** 
     * Decodes an ASN1Object according to the ASN.1 type the class implements.
     */
    public void decode(ASN1Object obj) throws CodingException {
	try {
	    SEQUENCE obo = (SEQUENCE) obj;
	    super.decode(obo.getComponentAt(0));
	    blinding_factor=(BigInteger)((INTEGER)obo.getComponentAt(1)).getValue();
	    oorder=new OpenOrder();
	    oorder.decode(obo.getComponentAt(2));
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    throw new CodingException();
	}

    }
    
    /**
     * Creates an ASN1Object according to the ASN.1 type the class implements.
     */
    public ASN1Object toASN1Object() throws CodingException {
	SEQUENCE obo=new SEQUENCE();
	obo.addComponent(super.toASN1Object());
	obo.addComponent(new INTEGER(blinding_factor));
	obo.addComponent(oorder.toASN1Object());
	return obo;
    }
    
    /**
     * The private information used to mask the blinded hash
     */
    protected BigInteger blinding_factor;

   /**
     * The enclosed open order
     */
    protected OpenOrder oorder;

}