package um.di.lmf.enote;

import iaik.asn1.*;
import java.math.*;
import java.util.*;
import java.security.*;
import iaik.utils.*;

import um.di.lmf.*;

/**
 * This class represents a digital cash note note order, disclosing all the commitments for the withdrawal protocol
 */

public class OpenOrder extends Order implements ASN1Type {
    
    /**
     * This is the constructor that should be used before building an object with decode.
     */
    public OpenOrder () {
    }

    /**
     * This constructor builds an open order from scratch.
     */
    public OpenOrder (String id, int amount, int npairs, iaik.x509.X509Certificate bank_cert) throws java.security.NoSuchAlgorithmException {
	SecureRandom sr=SecureRandom.getInstance("SHA1PRNG");
	sr.setSeed(Calendar.getInstance().get(Calendar.MILLISECOND ));
	
	MessageDigest md=MessageDigest.getInstance("SHA");
	
	this.amount=amount;
	this.npairs=npairs;
	
	serial=new BigInteger(160,sr);
	
	byte[] idbytes = id.getBytes();
	
	idhalf1=new byte[npairs][];
	idhalf2=new byte[npairs][];
	commit1=new byte[npairs][];
	commit2=new byte[npairs][];
	priv_random1=new BigInteger[npairs];
	priv_random2=new BigInteger[npairs];
	pub_random1=new BigInteger[npairs];
	pub_random2=new BigInteger[npairs];
	
	for (int i=0; i< npairs; i++) {
	    idhalf1[i]=new byte[idbytes.length];
	    sr.nextBytes(idhalf1[i]);
	    idhalf2[i]=new byte[idbytes.length];
	    CryptoUtils.xorBlock(idbytes,idhalf1[i],idhalf2[i]);
	    priv_random1[i]=new BigInteger(160,sr);
	    priv_random2[i]=new BigInteger(160,sr);
	    pub_random1[i]=new BigInteger(160,sr);
	    pub_random2[i]=new BigInteger(160,sr);
	    md.reset();
	    md.update(idhalf1[i]);
	    md.update(priv_random1[i].toByteArray());
	    md.update(pub_random1[i].toByteArray());
	    commit1[i]=md.digest();
	    md.reset();
	    md.update(idhalf2[i]);
	    md.update(priv_random2[i].toByteArray());
	    md.update(pub_random2[i].toByteArray());
	    commit2[i]=md.digest();
	}
	
	this.bank_cert=bank_cert;
    }

    /**
     * Returns one half of the ID
     */
    public byte[] getIDHalf(int i,boolean bit) {
	if (!bit)
	    return idhalf1[i];
	else
	    return idhalf2[i];
    }

   /**
     * Returns one of the commitment secret randoms
     */
    public BigInteger getPrivRandom(int i,boolean bit) {
	if (!bit)
	    return priv_random1[i];
	else
	    return priv_random2[i];
    }

    /** 
     * Decodes an ASN1Object according to the ASN.1 type the class implements.
     */
    public void decode(ASN1Object obj) throws CodingException {
	SEQUENCE oorder = (SEQUENCE) obj;

	try {
	    super.decode(oorder.getComponentAt(0));

	    idhalf1=new byte[npairs][];
	    idhalf2=new byte[npairs][];
	    priv_random1=new BigInteger[npairs];
	    priv_random2=new BigInteger[npairs];
	    
	    SEQUENCE pairs = (SEQUENCE) oorder.getComponentAt(1);
	    for (int i=0; i<npairs; i++) {
		SEQUENCE pair = (SEQUENCE) pairs.getComponentAt(i);
		idhalf1[i]=(byte []) ((OCTET_STRING) pair.getComponentAt(0)).getValue();
		idhalf2[i]=(byte []) ((OCTET_STRING) pair.getComponentAt(1)).getValue();
		priv_random1[i]=(BigInteger) ((INTEGER) pair.getComponentAt(2)).getValue();
		priv_random2[i]=(BigInteger) ((INTEGER) pair.getComponentAt(3)).getValue();
	    };

	}
	catch (ArrayIndexOutOfBoundsException e) {
	    throw new CodingException();
	}
    }

    /**
     * Creates an ASN1Object of the enclosed Order object
     */

    public ASN1Object toOrderASN1Object() throws CodingException {
	return super.toASN1Object();
    }

    /**
     * Creates an ASN1Object according to the ASN.1 type the class implements.
     */
    public ASN1Object toASN1Object() throws CodingException {
	SEQUENCE oorder=new SEQUENCE();

	oorder.addComponent(super.toASN1Object());

	SEQUENCE pairs = new SEQUENCE();
	for (int i=0; i<npairs; i++) {
	    SEQUENCE pair = new SEQUENCE();
	    pair.addComponent(new OCTET_STRING(idhalf1[i]));
	    pair.addComponent(new OCTET_STRING(idhalf2[i]));
	    pair.addComponent(new INTEGER(priv_random1[i]));
	    pair.addComponent(new INTEGER(priv_random2[i]));
	    pairs.addComponent(pair);
	}
	oorder.addComponent(pairs);

	return oorder;
    }

    /**
     * The private information for the bit commitments: one id half
     */
    protected byte[][] idhalf1;

    /**
     * The private information for the bit commitments: one id half
     */
    protected byte[][] idhalf2;

   /**
     * The private information for the bit commitments: one private random
     */
    protected BigInteger[] priv_random1;

   /**
     * The private information for the bit commitments: one private random
     */
    protected BigInteger[] priv_random2;

}