package um.di.lmf.security;

import um.di.lmf.security.*;

import java.math.*;
import java.security.*;
import java.security.interfaces.*;
import iaik.asn1.*;
import iaik.utils.*;

/**
 * This class implements four procedures related to the RSA Blind Signature
 */

public class RSABlindSignature {

    /**
     * This method can be used to obtain a blinded hash, to be signed by the public key owner.
     */
    public static byte[] blind(byte[] data, BigInteger blinding_factor, RSAPublicKey pubKey) throws RSABlindSignatureException {
	try {
	    blinding_factor=blinding_factor.abs();
	    MessageDigest md = MessageDigest.getInstance("SHA");
	    md.update(data);
	    byte[] hash=md.digest();
	    
	    BigInteger mask=blinding_factor.modPow(pubKey.getPublicExponent(),pubKey.getModulus());
	    
	    return (new BigInteger(hash).abs().multiply(mask).mod(pubKey.getModulus())).toByteArray();
       	}
	catch (Exception e) {
	    throw new RSABlindSignatureException();
	}
    }

    /**
     * This method allows the private key owner to sign a previously blinded hash.
     */
    public static byte[]sign(byte[] blinded_hash, RSAPrivateKey privKey) throws RSABlindSignatureException {
	try {
	    BigInteger hash=new BigInteger(blinded_hash).abs();
	    byte[]signature=hash.modPow(privKey.getPrivateExponent(),privKey.getModulus()).toByteArray();
	    return signature;
	}
	catch (Exception e) {
	    throw new RSABlindSignatureException();
	}
    }
    
    /**
     * This method permits recovering a proper signature, removing the blinding from 
     * the result of the sign function.
     */
    public static byte[]unblind(byte[] blinded_sig, BigInteger blinding_factor, RSAPublicKey pubKey) throws RSABlindSignatureException {
	try {
	    blinding_factor=blinding_factor.abs();
	    BigInteger bsig=new BigInteger(blinded_sig).abs();
	    byte[]signature=bsig.multiply(blinding_factor.modInverse(pubKey.getModulus())).mod(pubKey.getModulus()).toByteArray();
	    return signature;
	}
	catch (Exception e) {
	    throw new RSABlindSignatureException();
	}
    }

    /**
     * This method permits verifying an unblinded signature.
     */
    public static boolean verify(byte [] data, byte[] signature, RSAPublicKey pubKey) throws RSABlindSignatureException {
	try{
	    MessageDigest md = MessageDigest.getInstance("SHA");
	    md.update(data);
	    byte[] hash=md.digest();
	    
	    BigInteger sig=new BigInteger(signature).abs();
	    BigInteger recov_hash_i=sig.modPow(pubKey.getPublicExponent(),pubKey.getModulus());

	    BigInteger hash_i=new BigInteger(hash).abs();

	    return(hash_i.equals(recov_hash_i));
	}
	catch (Exception e) {
	    throw new RSABlindSignatureException();
	}
    }
}